package com.example.demo.videos.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(VideoAdminController.URL)
public class VideoAdminController {
    public static final String URL = Constants.ADMIN_PREFIX + "/video";
    private static final String VIDEO_VIEW = "video";
    private static final String VIDEO_EDIT_VIEW = "video-edit";
    private static final String VIDEO_ATTRIBUTE = "video";

    private static final String PAGE_ATTRIBUTE = "page";

    private final UserService userService;
    private final VideoService videoService;
    private final ModelMapper modelMapper;

    public VideoAdminController(UserService user, VideoService video, ModelMapper model) {
        userService = user;
        videoService = video;
        modelMapper = model;
    }

    private VideoDto toDto(VideoEntity entity) {
        return modelMapper.map(entity, VideoDto.class);
    }

    private VideoEntity toEntity(VideoDto dto) {
        final VideoEntity entity = modelMapper.map(dto, VideoEntity.class);
        entity.setUser(userService.get(dto.getUserId()));
        return entity;
    }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attrib = PageAttributesMapper.toAttributes(
                videoService.getAll(0L, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attrib);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return VIDEO_VIEW;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(VIDEO_ATTRIBUTE, toDto(videoService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return VIDEO_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = VIDEO_ATTRIBUTE) @Valid VideoDto video,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return VIDEO_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        videoService.update(id, toEntity(video));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        videoService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
