package com.example.demo.videos.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.comments.api.CommentDto;
import com.example.demo.comments.api.CommentUserVideoTitleDto;
import com.example.demo.comments.model.CommentEntity;
import com.example.demo.comments.service.CommentService;
import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.likes.api.LikeDto;
import com.example.demo.likes.api.LikeTitleDto;
import com.example.demo.likes.model.LikeEntity;
import com.example.demo.likes.service.LikeService;
import com.example.demo.videos.model.VideoEntity;
import com.example.demo.videos.service.VideoService;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

@Controller
@RequestMapping(VideoUserController.URL)
public class VideoUserController {
    public static final String URL = "/video";
    private static final String VIDEO_VIEW = "user-video";
    private static final String VIDEO_EDIT_VIEW = "video-edit";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String VIDEO_ATTRIBUTE = "video";

    private static final String LIKE_VIEW = "video-like";

    private static final String COMMENT_VIEW = "comment-user";
    private static final String COMMENT_EDIT_VIEW = "comment-edit-user";
    private static final String COMMENT_ATTRIBUTE = "comment";
    private static final String COMMENT_ADD_VIEW = "comment-edit-add";

    private final UserService userService;
    private final VideoService videoService;
    private final ModelMapper modelMapper;
    private final LikeService likeService;
    private final CommentService commentService;

    public VideoUserController(UserService user, VideoService video, ModelMapper model, LikeService like,
            CommentService comment) {
        userService = user;
        videoService = video;
        modelMapper = model;
        likeService = like;
        commentService = comment;
    }

    private VideoDto toDto(VideoEntity entity) {
        return modelMapper.map(entity, VideoDto.class);
    }

    private LikeTitleDto toDto(LikeEntity entity) {
        LikeTitleDto dto = new LikeTitleDto();
        dto.setId(entity.getId());
        dto.setUserId(entity.getUser().getId());
        dto.setVideoId(entity.getVideo().getId());
        dto.setTitle(entity.getVideo().getTitle());
        dto.setUser(entity.getUser().getLogin());
        return dto;
    }

    private CommentUserVideoTitleDto toDto(CommentEntity entity) {
        CommentUserVideoTitleDto dto = new CommentUserVideoTitleDto();
        dto.setId(entity.getId());
        dto.setText(entity.getText());
        dto.setUserId(entity.getUser().getId());
        dto.setVideoId(entity.getVideo().getId());
        dto.setUser(entity.getUser().getLogin());
        dto.setTitle(entity.getVideo().getTitle());
        return dto;
    }

    private VideoEntity toEntity(VideoDto dto) {
        final VideoEntity entity = modelMapper.map(dto, VideoEntity.class);
        entity.setUser(userService.get(dto.getUserId()));
        return entity;
    }

    private CommentEntity toEntity(CommentDto dto) {
        final CommentEntity entity = modelMapper.map(dto, CommentEntity.class);
        entity.setUser(userService.get(dto.getUserId()));
        entity.setVideo(videoService.get(dto.getVideoId()));
        return entity;
    }

    private LikeEntity toEntity(LikeDto dto) {
        final LikeEntity entity = modelMapper.map(dto, LikeEntity.class);
        entity.setUser(userService.get(dto.getUserId()));
        entity.setVideo(videoService.get(dto.getVideoId()));
        return entity;
    }

    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Long userId = userService.getUserIdLong();
        final Map<String, Object> attrib = PageAttributesMapper.toAttributes(
                videoService.getAll(userId, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attrib);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return VIDEO_VIEW;
    }

    @GetMapping("/edit/")
    public String create(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final VideoDto dto = new VideoDto();
        dto.setUserId(userService.getUserIdLong());
        model.addAttribute(VIDEO_ATTRIBUTE, dto);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return VIDEO_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = VIDEO_ATTRIBUTE) @Valid VideoDto video,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return VIDEO_EDIT_VIEW;
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        videoService.create(toEntity(video));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(VIDEO_ATTRIBUTE, toDto(videoService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return VIDEO_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = VIDEO_ATTRIBUTE) @Valid VideoDto video,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return VIDEO_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        videoService.update(id, toEntity(video));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        videoService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/title/{id}")
    public String getName(@PathVariable(name = "id") Long id) {
        final VideoEntity video = videoService.get(id);
        return video.getTitle();
    }

    @GetMapping("/comment")
    public String getComments(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Long id = userService.getUserIdLong();
        final Map<String, Object> attrib = PageAttributesMapper.toAttributes(
                commentService.getAllByUser(id, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attrib);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return COMMENT_VIEW;
    }

    @GetMapping("/comment/edit/add/{videoId}")
    public String createComment(@PathVariable(name = "videoId") Long videoId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final CommentDto dto = new CommentDto();
        dto.setUserId(userService.getUserIdLong());
        dto.setVideoId(videoId);
        model.addAttribute(COMMENT_ATTRIBUTE, dto);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return COMMENT_ADD_VIEW;
    }

    @PostMapping("/comment/edit/add")
    public String createComment(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = VIDEO_ATTRIBUTE) @Valid CommentDto comment,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return COMMENT_ADD_VIEW;
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        commentService.create(toEntity(comment));
        return Constants.REDIRECT_VIEW + "/list/video";
    }

    @GetMapping("/comment/edit/{id}")
    public String updateComment(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(COMMENT_ATTRIBUTE, toDto(commentService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return COMMENT_EDIT_VIEW;
    }

    @PostMapping("/comment/edit/{id}")
    public String updateComment(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = VIDEO_ATTRIBUTE) @Valid CommentDto comment,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return VIDEO_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        commentService.update(id, toEntity(comment));
        return Constants.REDIRECT_VIEW + URL + "/comment";
    }

    @PostMapping("/comment/delete/{id}")
    public String deleteComment(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        commentService.delete(id);
        return Constants.REDIRECT_VIEW + URL + "/comment";
    }

    @GetMapping("/like")
    public String getLikes(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Long id = userService.getUserIdLong();
        final Map<String, Object> attrib = PageAttributesMapper.toAttributes(
                likeService.getAllByUser(id, page, Constants.DEFUALT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attrib);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return LIKE_VIEW;
    }

    @PostMapping("/like/{videoId}")
    public String addLike(@PathVariable(name = "videoId") Long videoId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        final Long userid = userService.getUserIdLong();
        final LikeDto dto = new LikeDto();
        dto.setUserId(userid);
        dto.setVideoId(videoId);
        likeService.create(toEntity(dto));
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        return Constants.REDIRECT_VIEW + "/list/video";
    }

    @PostMapping("/dislike1/{id}")
    public String deleteLike1(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        final Long userid = userService.getUserIdLong();
        likeService.deleteByUserIdAndVideoId(userid, id);
        return Constants.REDIRECT_VIEW + "/list/video";
    }

    @PostMapping("/dislike2/{id}")
    public String deleteLike2(@PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        final Long userid = userService.getUserIdLong();
        likeService.deleteByUserIdAndVideoId(userid, id);
        return Constants.REDIRECT_VIEW + URL + "/like";
    }
}
